#property copyright "ArteInVolo"
#property version "1.00"

#property indicator_separate_window 
#property indicator_buffers 1
#property indicator_plots 1

#property indicator_type1 DRAW_LINE
#property indicator_color1 clrBlue
#property indicator_style1 STYLE_SOLID
#property indicator_width1 1
#property indicator_label1 "Index"

#include "PCA.mqh"

// *** Input *** //

input int InpDepth = 1000;
input int InpOosDepth = 0;
input int InpIterations = 1000;
input bool InpUseMinimum = true;
input bool InpUseCorrelation = true;
input double InpPrecision = 0.0001;
input ENUM_TIMEFRAMES InpPeriod = PERIOD_CURRENT;
input string InpUnit = "USD";
input string InpSymbols = "AUDUSD,EURUSD,GBPUSD,USDCHF,USDJPY,USDCAD,NZDUSD,USDSEK,USDSGD,USDMXN,USDNOK";

// *** Globals *** //

double iChart[];
int iOrder, iDepth, iSourceDepth;
SSeries iSeries[], iCovs[], iCovsVector[], iCovsEigen[], iCors[], iCorsVector[], iCorsEigen[];

// *** Implementation *** //

int OnInit()
{
    SetIndexBuffer(0, iChart, INDICATOR_DATA);
    ArraySetAsSeries(iChart, true);

    IndicatorSetInteger(INDICATOR_DIGITS, _Digits);
    IndicatorSetString(INDICATOR_SHORTNAME, "PCA");
    
    iSourceDepth = InpDepth;
    iDepth = iSourceDepth + InpOosDepth;
    iOrder = ListSymbols(iSeries, GetSymbolNames(InpSymbols, InpUnit));

    SetupMatrix(iCovs, iOrder, iOrder);
    SetupMatrix(iCovsEigen, iOrder, iOrder);
    SetupMatrix(iCovsVector, iOrder, iDepth);
    SetupMatrix(iCors, iOrder, iOrder);
    SetupMatrix(iCorsEigen, iOrder, iOrder);
    SetupMatrix(iCorsVector, iOrder, iDepth);
    SetupMatrix(iSeries, iOrder, iDepth);
    
    ClearMatrix(iCovs, iOrder, iOrder);
    ClearMatrix(iCovsEigen, iOrder, iOrder);
    ClearMatrix(iCovsVector, iOrder, iDepth);
    ClearMatrix(iCors, iOrder, iOrder);
    ClearMatrix(iCorsEigen, iOrder, iOrder);
    ClearMatrix(iCorsVector, iOrder, iDepth);
    ClearMatrix(iSeries, iOrder, iDepth);

    SetupPCA(iOrder, iOrder);
    
    EventSetTimer(1);

    return(INIT_SUCCEEDED);
}

void OnDeinit(const int reason)
{
    HidePanel();
}

void OnTimer()
{
    Display();
}

int OnCalculate(
    const int bars,
    const int counted,
    const datetime &time[],
    const double &open[],
    const double& high[],
    const double& low[],
    const double &close[],
    const long &tickVolume[],
    const long &volume[],
    const int &spread[])
{
    return(bars);
}

void Display()
{
    if (Synchronize(iSeries, InpPeriod, iOrder, iDepth))
    {
        int index = 0;
        datetime oosTime = GetLastTime(iSeries, InpPeriod, iOrder) - PeriodSeconds(InpPeriod) * InpOosDepth;
    
        ClearMatrix(iCovsEigen, iOrder, iOrder);
        ClearMatrix(iCovsVector, iOrder, iDepth);
        GetBaseMatrix(iSeries, iOrder, iSourceDepth);
        GetCovMatrix(iSeries, iCovs, iOrder, iSourceDepth);
        GetEigenValues(iCovs, iCovsEigen, iCovsVector, iOrder, InpPrecision, InpIterations);
        int vectorIndexCov = GetOptimalVector(iCovsEigen, iOrder, InpUseMinimum);
        GetIndexValues(iSeries, iCovsVector, iOrder, iDepth, vectorIndexCov);
        SetLine("OOS", oosTime);

        for (int k = iDepth - 1; k > 0; k--)
        {
            iChart[index] = iCovsVector[0].mSyncCloses[k];
            index++;
        }

        if (InpUseCorrelation)
        {
            ClearMatrix(iCorsEigen, iOrder, iOrder);
            ClearMatrix(iCorsVector, iOrder, iDepth);
            GetCorMatrix(iCovs, iCors, iOrder);
            GetEigenValues(iCors, iCorsEigen, iCorsVector, iOrder, InpPrecision, InpIterations);
            int vectorIndexCor = GetOptimalVector(iCorsEigen, iOrder, InpUseMinimum);
            ShowVector(iSeries, iCorsVector, iOrder, vectorIndexCor);
        }
        else
        {
            ShowVector(iSeries, iCovsVector, iOrder, vectorIndexCov);
        }
    }

    ChartRedraw();
}